#include <boost/program_options.hpp>
namespace po = boost::program_options;
#include <boost/filesystem/path.hpp>
namespace fs = boost::filesystem;

#include <string>
#include <iostream>

#include "alg/mussa.hpp"
#include "alg/parse_options.hpp"

MussaOptions::MussaOptions()
  : useGUI(true),
    runAsPythonInterpeter(false)
{
}

void initialize_mussa(MussaOptions& opts, int argc, char **argv)
{
  po::options_description options("Mussa options");
  po::positional_options_description pos_options;
  pos_options.add("run-analysis", -1);

  options.add_options()
    ("help", "help message")
    ("run-analysis,p", po::value<std::string>(), 
     "run an analysis defined by the mussa parameter file")
    ("view-analysis", po::value<std::string>(),
     "load a previously run analysis")
    ("motifs", po::value<std::string>(),
     "annotate analysis with motifs from this file")
    ("no-gui", "terminate without running an analysis")
    ("python", "launch as a python interpreter")
  ;

  po::variables_map vm;
  po::store(po::command_line_parser(argc, argv).options(options).positional(pos_options).run(), vm);
  po::notify(vm);

  if (vm.count("help")) {
    std::cout << options << std::endl;
    return;
  }
  
  opts.analysis = Mussa::init();

  // currently we can only have one analysis loaded, so 
  // running trumps viewing.
  if (vm.count("run-analysis")) {
    fs::path analysis_path(vm["run-analysis"].as< std::string >(), fs::native);
    opts.analysis->load_mupa_file( analysis_path );
    std::cout << "I apologize for blocking the gui while running the analysis"
              << std::endl;
    opts.analysis->analyze();
    if (opts.analysis->is_dirty())
      opts.analysis->save();
  }
  else if (vm.count("view-analysis")) {
    fs::path analysis_path(vm["view-analysis"].as< std::string >(), fs::native);
    opts.analysis->load( analysis_path );
  }
  if (vm.count("motifs") and opts.analysis) {
    fs::path motif_path(vm["motifs"].as< std::string >(), fs::native);
    opts.analysis->load_motifs( motif_path );
  }
  if (vm.count("no-gui")) {
    opts.useGUI=false;
  }
  if (vm.count("python")) {
    opts.runAsPythonInterpeter = true;
  }
}
