#ifndef TESTMOTIFELEMENT_HPP_
#define TESTMOTIFELEMENT_HPP_
#include <QtGui>
#include <QtTest/QtTest>

#include "alg/sequence.hpp"
#include "qui/motif_editor/MotifElement.hpp"

class TestMotifElement : public QObject
{
  Q_OBJECT

private slots:

  void testSimpleConstructor() {
    MotifElement me;
    Color default_color(1.0, 0.0, 0.0, 1.0);
    QVERIFY(me.getColor() == default_color);
    QVERIFY(me.isEnabled() == true);
    QVERIFY(me.isEmpty() == true);
    QVERIFY(me.getSequence() == "");
    QVERIFY(me.getName() == "");    
  }

  void testSequenceConstructor() {
    Sequence s("AAAAAGGGGGG");
    Color white(1.0, 1.0, 1.0);
    
    MotifElement me(s, white);
    QVERIFY(me.getSequence() == s.get_sequence());
    QVERIFY(me.getColor() == white);
    QVERIFY(me.isEmpty() == false);
    QVERIFY(me.getName() == "");    
  }
  
  void testColorConverters() {
    Sequence s("AAAAAGGGGGG");
    Color white(1.0, 1.0, 1.0);
    QColor qwhite(255,255,255);
    Color red(1.0, 0.0, 0.0);
    QColor qred(255,0,0);
    
    MotifElement me(s, white);
    QVERIFY(me.getColor() == white);
    QVERIFY(me.getQColor() == qwhite);
    
    me.setColor(red);
    QVERIFY(me.getColor() == red);
    QVERIFY(me.getQColor() == qred);    
  }
  
  void testSequenceText() {
    std::string seq_text("AAAAGGGGG");
    QString qs(seq_text.c_str());
    Sequence s(seq_text);
    MotifElement me(s);
    
    QVERIFY(me.getSequence() == s);
    QVERIFY(me.getSequence() == seq_text);
    QVERIFY(me.getSequenceText() == qs);
    
    std::string seq_text2("GGGGTTTTT");
    QString qs2(seq_text2.c_str());
    me.setSequence(seq_text2);
    QVERIFY(me.getSequence() == seq_text2);
    QVERIFY(me.getSequenceText() == qs2);    
  }
  
  void testSequenceName() {
    Sequence s("AAAAGGGG");
    MotifElement me(s);
    me.setName("hello");
    QVERIFY(me.getName() == "hello");
  }
};

#endif /*TESTMOTIFELEMENT_HPP_*/
